@@ -69,6 +69,29 @@ class Settings(object):
         "seek_down":            -60,
         "seek_right":           5,
         "seek_left":            -5,
+        "shader_pack_enable":   True,
+        "shader_pack_custom":   False,
+        "shader_pack_remember": True,
+        "shader_pack_profile":  None,
+        "svp_enable":           False,
+        "svp_url":              "http://127.0.0.1:9901/",
+        "svp_socket":           None,
+        "sanitize_output":      True,
+        "write_logs":           False,
+        "playback_timeout":     30,
+        "sync_max_delay_speed": 50,
+        "sync_max_delay_skip":  300,
+        "sync_method_thresh":   2000,
+        "sync_speed_time":      1000,
+        "sync_speed_attempts":  3,
+        "sync_attempts":        5,
+        "sync_revert_seek":     True,
+        "sync_osd_message":     True,
+        "screenshot_menu":      True,
+        "check_updates":        True,
+        "notify_updates":       True,
+        "lang":                 None,
+        "desktop_scale":        1.0,
     }
 
     def __getattr__(self, name):
